/**
 ******************************************************************************
 *
 * @file       uavobjecttemplate.cpp
 * @author     The OpenPilot Team, http://www.openpilot.org Copyright (C) 2010.
 * @brief      Template for an uavobject in java
 *             This is a autogenerated file!! Do not modify and expect a result.
 *             Settings for the @ref Attitude module used on CopterControl
 *
 * @see        The GNU Public License (GPL) Version 3
 *
 *****************************************************************************/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.openpilot.uavtalk.uavobjects;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.openpilot.uavtalk.UAVObjectManager;
import org.openpilot.uavtalk.UAVObject;
import org.openpilot.uavtalk.UAVDataObject;
import org.openpilot.uavtalk.UAVObjectField;

/**
Settings for the @ref Attitude module used on CopterControl

generated from attitudesettings.xml
 **/
public class AttitudeSettings extends UAVDataObject {

	public AttitudeSettings() {
		super(OBJID, ISSINGLEINST, ISSETTINGS, NAME);
		
		List<UAVObjectField> fields = new ArrayList<UAVObjectField>();
		

		List<String> GyroGainElemNames = new ArrayList<String>();
		GyroGainElemNames.add("0");
		fields.add( new UAVObjectField("GyroGain", "(rad/s)/lsb", UAVObjectField.FieldType.FLOAT32, GyroGainElemNames, null) );

		List<String> AccelKpElemNames = new ArrayList<String>();
		AccelKpElemNames.add("0");
		fields.add( new UAVObjectField("AccelKp", "channel", UAVObjectField.FieldType.FLOAT32, AccelKpElemNames, null) );

		List<String> AccelKiElemNames = new ArrayList<String>();
		AccelKiElemNames.add("0");
		fields.add( new UAVObjectField("AccelKi", "channel", UAVObjectField.FieldType.FLOAT32, AccelKiElemNames, null) );

		List<String> YawBiasRateElemNames = new ArrayList<String>();
		YawBiasRateElemNames.add("0");
		fields.add( new UAVObjectField("YawBiasRate", "channel", UAVObjectField.FieldType.FLOAT32, YawBiasRateElemNames, null) );

		List<String> AccelBiasElemNames = new ArrayList<String>();
		AccelBiasElemNames.add("X");
		AccelBiasElemNames.add("Y");
		AccelBiasElemNames.add("Z");
		fields.add( new UAVObjectField("AccelBias", "lsb", UAVObjectField.FieldType.INT16, AccelBiasElemNames, null) );

		List<String> GyroBiasElemNames = new ArrayList<String>();
		GyroBiasElemNames.add("X");
		GyroBiasElemNames.add("Y");
		GyroBiasElemNames.add("Z");
		fields.add( new UAVObjectField("GyroBias", "deg/s * 100", UAVObjectField.FieldType.INT16, GyroBiasElemNames, null) );

		List<String> BoardRotationElemNames = new ArrayList<String>();
		BoardRotationElemNames.add("Roll");
		BoardRotationElemNames.add("Pitch");
		BoardRotationElemNames.add("Yaw");
		fields.add( new UAVObjectField("BoardRotation", "deg", UAVObjectField.FieldType.INT16, BoardRotationElemNames, null) );

		List<String> ZeroDuringArmingElemNames = new ArrayList<String>();
		ZeroDuringArmingElemNames.add("0");
		List<String> ZeroDuringArmingEnumOptions = new ArrayList<String>();
		ZeroDuringArmingEnumOptions.add("FALSE");
		ZeroDuringArmingEnumOptions.add("TRUE");
		fields.add( new UAVObjectField("ZeroDuringArming", "channel", UAVObjectField.FieldType.ENUM, ZeroDuringArmingElemNames, ZeroDuringArmingEnumOptions) );

		List<String> BiasCorrectGyroElemNames = new ArrayList<String>();
		BiasCorrectGyroElemNames.add("0");
		List<String> BiasCorrectGyroEnumOptions = new ArrayList<String>();
		BiasCorrectGyroEnumOptions.add("FALSE");
		BiasCorrectGyroEnumOptions.add("TRUE");
		fields.add( new UAVObjectField("BiasCorrectGyro", "channel", UAVObjectField.FieldType.ENUM, BiasCorrectGyroElemNames, BiasCorrectGyroEnumOptions) );

		List<String> TrimFlightElemNames = new ArrayList<String>();
		TrimFlightElemNames.add("0");
		List<String> TrimFlightEnumOptions = new ArrayList<String>();
		TrimFlightEnumOptions.add("NORMAL");
		TrimFlightEnumOptions.add("START");
		TrimFlightEnumOptions.add("LOAD");
		fields.add( new UAVObjectField("TrimFlight", "channel", UAVObjectField.FieldType.ENUM, TrimFlightElemNames, TrimFlightEnumOptions) );


		// Compute the number of bytes for this object
		int numBytes = 0;
		ListIterator<UAVObjectField> li = fields.listIterator();
		while(li.hasNext()) {
			numBytes += li.next().getNumBytes();
		}
		NUMBYTES = numBytes;

		// Initialize object
		initializeFields(fields, ByteBuffer.allocate(NUMBYTES), NUMBYTES);
		// Set the default field values
		setDefaultFieldValues();
		// Set the object description
		setDescription(DESCRIPTION);
	}

	/**
	 * Create a Metadata object filled with default values for this object
	 * @return Metadata object with default values
	 */
	public Metadata getDefaultMetadata() {
		UAVObject.Metadata metadata = new UAVObject.Metadata();
    	metadata.flags =
		    UAVObject.Metadata.AccessModeNum(UAVObject.AccessMode.ACCESS_READWRITE) << UAVOBJ_ACCESS_SHIFT |
		    UAVObject.Metadata.AccessModeNum(UAVObject.AccessMode.ACCESS_READWRITE) << UAVOBJ_GCS_ACCESS_SHIFT |
		    1 << UAVOBJ_TELEMETRY_ACKED_SHIFT |
		    1 << UAVOBJ_GCS_TELEMETRY_ACKED_SHIFT |
		    UAVObject.Metadata.UpdateModeNum(UAVObject.UpdateMode.UPDATEMODE_ONCHANGE) << UAVOBJ_TELEMETRY_UPDATE_MODE_SHIFT |
		    UAVObject.Metadata.UpdateModeNum(UAVObject.UpdateMode.UPDATEMODE_ONCHANGE) << UAVOBJ_GCS_TELEMETRY_UPDATE_MODE_SHIFT;
    	metadata.flightTelemetryUpdatePeriod = 0;
    	metadata.gcsTelemetryUpdatePeriod = 0;
    	metadata.loggingUpdatePeriod = 0;
 
		return metadata;
	}

	/**
	 * Initialize object fields with the default values.
	 * If a default value is not specified the object fields
	 * will be initialized to zero.
	 */
	public void setDefaultFieldValues()
	{
		getField("GyroGain").setValue(0.42);
		getField("AccelKp").setValue(0.05);
		getField("AccelKi").setValue(0.0001);
		getField("YawBiasRate").setValue(1e-06);
		getField("AccelBias").setValue(0,0);
		getField("AccelBias").setValue(0,1);
		getField("AccelBias").setValue(0,2);
		getField("GyroBias").setValue(0,0);
		getField("GyroBias").setValue(0,1);
		getField("GyroBias").setValue(0,2);
		getField("BoardRotation").setValue(0,0);
		getField("BoardRotation").setValue(0,1);
		getField("BoardRotation").setValue(0,2);
		getField("ZeroDuringArming").setValue("TRUE");
		getField("BiasCorrectGyro").setValue("TRUE");
		getField("TrimFlight").setValue("NORMAL");

	}

	/**
	 * Create a clone of this object, a new instance ID must be specified.
	 * Do not use this function directly to create new instances, the
	 * UAVObjectManager should be used instead.
	 */
	public UAVDataObject clone(long instID) {
		// TODO: Need to get specific instance to clone
		try {
			AttitudeSettings obj = new AttitudeSettings();
			obj.initialize(instID, this.getMetaObject());
			return obj;
		} catch  (Exception e) {
			return null;
		}
	}

	/**
	 * Static function to retrieve an instance of the object.
	 */
	public AttitudeSettings GetInstance(UAVObjectManager objMngr, long instID)
	{
	    return (AttitudeSettings)(objMngr.getObject(AttitudeSettings.OBJID, instID));
	}

	// Constants
	protected static final long OBJID = 0xC307BC4Al;
	protected static final String NAME = "AttitudeSettings";
	protected static String DESCRIPTION = "Settings for the @ref Attitude module used on CopterControl";
	protected static final boolean ISSINGLEINST = 1 > 0;
	protected static final boolean ISSETTINGS = 1 > 0;
	protected static int NUMBYTES = 0;


}
