/**
 ******************************************************************************
 *
 * @file       uavobjecttemplate.cpp
 * @author     The OpenPilot Team, http://www.openpilot.org Copyright (C) 2010.
 * @brief      Template for an uavobject in java
 *             This is a autogenerated file!! Do not modify and expect a result.
 *             Status of the flight plan script
 *
 * @see        The GNU Public License (GPL) Version 3
 *
 *****************************************************************************/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.openpilot.uavtalk.uavobjects;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.openpilot.uavtalk.UAVObjectManager;
import org.openpilot.uavtalk.UAVObject;
import org.openpilot.uavtalk.UAVDataObject;
import org.openpilot.uavtalk.UAVObjectField;

/**
Status of the flight plan script

generated from flightplanstatus.xml
 **/
public class FlightPlanStatus extends UAVDataObject {

	public FlightPlanStatus() {
		super(OBJID, ISSINGLEINST, ISSETTINGS, NAME);
		
		List<UAVObjectField> fields = new ArrayList<UAVObjectField>();
		

		List<String> ErrorFileIDElemNames = new ArrayList<String>();
		ErrorFileIDElemNames.add("0");
		fields.add( new UAVObjectField("ErrorFileID", "", UAVObjectField.FieldType.UINT32, ErrorFileIDElemNames, null) );

		List<String> ErrorLineNumElemNames = new ArrayList<String>();
		ErrorLineNumElemNames.add("0");
		fields.add( new UAVObjectField("ErrorLineNum", "", UAVObjectField.FieldType.UINT32, ErrorLineNumElemNames, null) );

		List<String> DebugElemNames = new ArrayList<String>();
		DebugElemNames.add("0");
		DebugElemNames.add("1");
		fields.add( new UAVObjectField("Debug", "", UAVObjectField.FieldType.FLOAT32, DebugElemNames, null) );

		List<String> StatusElemNames = new ArrayList<String>();
		StatusElemNames.add("0");
		List<String> StatusEnumOptions = new ArrayList<String>();
		StatusEnumOptions.add("Stopped");
		StatusEnumOptions.add("Running");
		StatusEnumOptions.add("Error");
		fields.add( new UAVObjectField("Status", "", UAVObjectField.FieldType.ENUM, StatusElemNames, StatusEnumOptions) );

		List<String> ErrorTypeElemNames = new ArrayList<String>();
		ErrorTypeElemNames.add("0");
		List<String> ErrorTypeEnumOptions = new ArrayList<String>();
		ErrorTypeEnumOptions.add("None");
		ErrorTypeEnumOptions.add("VMInitError");
		ErrorTypeEnumOptions.add("Exception");
		ErrorTypeEnumOptions.add("IOError");
		ErrorTypeEnumOptions.add("DivByZero");
		ErrorTypeEnumOptions.add("AssertError");
		ErrorTypeEnumOptions.add("AttributeError");
		ErrorTypeEnumOptions.add("ImportError");
		ErrorTypeEnumOptions.add("IndexError");
		ErrorTypeEnumOptions.add("KeyError");
		ErrorTypeEnumOptions.add("MemoryError");
		ErrorTypeEnumOptions.add("NameError");
		ErrorTypeEnumOptions.add("SyntaxError");
		ErrorTypeEnumOptions.add("SystemError");
		ErrorTypeEnumOptions.add("TypeError");
		ErrorTypeEnumOptions.add("ValueError");
		ErrorTypeEnumOptions.add("StopIteration");
		ErrorTypeEnumOptions.add("Warning");
		ErrorTypeEnumOptions.add("UnknownError");
		fields.add( new UAVObjectField("ErrorType", "", UAVObjectField.FieldType.ENUM, ErrorTypeElemNames, ErrorTypeEnumOptions) );


		// Compute the number of bytes for this object
		int numBytes = 0;
		ListIterator<UAVObjectField> li = fields.listIterator();
		while(li.hasNext()) {
			numBytes += li.next().getNumBytes();
		}
		NUMBYTES = numBytes;

		// Initialize object
		initializeFields(fields, ByteBuffer.allocate(NUMBYTES), NUMBYTES);
		// Set the default field values
		setDefaultFieldValues();
		// Set the object description
		setDescription(DESCRIPTION);
	}

	/**
	 * Create a Metadata object filled with default values for this object
	 * @return Metadata object with default values
	 */
	public Metadata getDefaultMetadata() {
		UAVObject.Metadata metadata = new UAVObject.Metadata();
    	metadata.flags =
		    UAVObject.Metadata.AccessModeNum(UAVObject.AccessMode.ACCESS_READWRITE) << UAVOBJ_ACCESS_SHIFT |
		    UAVObject.Metadata.AccessModeNum(UAVObject.AccessMode.ACCESS_READWRITE) << UAVOBJ_GCS_ACCESS_SHIFT |
		    0 << UAVOBJ_TELEMETRY_ACKED_SHIFT |
		    0 << UAVOBJ_GCS_TELEMETRY_ACKED_SHIFT |
		    UAVObject.Metadata.UpdateModeNum(UAVObject.UpdateMode.UPDATEMODE_PERIODIC) << UAVOBJ_TELEMETRY_UPDATE_MODE_SHIFT |
		    UAVObject.Metadata.UpdateModeNum(UAVObject.UpdateMode.UPDATEMODE_MANUAL) << UAVOBJ_GCS_TELEMETRY_UPDATE_MODE_SHIFT;
    	metadata.flightTelemetryUpdatePeriod = 2000;
    	metadata.gcsTelemetryUpdatePeriod = 0;
    	metadata.loggingUpdatePeriod = 0;
 
		return metadata;
	}

	/**
	 * Initialize object fields with the default values.
	 * If a default value is not specified the object fields
	 * will be initialized to zero.
	 */
	public void setDefaultFieldValues()
	{
		getField("Debug").setValue(0,0);
		getField("Debug").setValue(0,1);
		getField("Status").setValue("Stopped");
		getField("ErrorType").setValue("None");

	}

	/**
	 * Create a clone of this object, a new instance ID must be specified.
	 * Do not use this function directly to create new instances, the
	 * UAVObjectManager should be used instead.
	 */
	public UAVDataObject clone(long instID) {
		// TODO: Need to get specific instance to clone
		try {
			FlightPlanStatus obj = new FlightPlanStatus();
			obj.initialize(instID, this.getMetaObject());
			return obj;
		} catch  (Exception e) {
			return null;
		}
	}

	/**
	 * Static function to retrieve an instance of the object.
	 */
	public FlightPlanStatus GetInstance(UAVObjectManager objMngr, long instID)
	{
	    return (FlightPlanStatus)(objMngr.getObject(FlightPlanStatus.OBJID, instID));
	}

	// Constants
	protected static final long OBJID = 0x2206EE46l;
	protected static final String NAME = "FlightPlanStatus";
	protected static String DESCRIPTION = "Status of the flight plan script";
	protected static final boolean ISSINGLEINST = 1 > 0;
	protected static final boolean ISSETTINGS = 0 > 0;
	protected static int NUMBYTES = 0;


}
